@@ -24,6 +24,7 @@ var _ = Describe("Legacy Options", func() {
 			legacyOpts.LegacyUpstreams.SSLUpstreamInsecureSkipVerify = true
 			legacyOpts.LegacyUpstreams.Upstreams = []string{"http://foo.bar/baz", "file:///var/lib/website#/bar", "static://204"}
 			legacyOpts.LegacyProvider.ClientID = "oauth-proxy"
+			legacyOpts.LegacyUpstreams.DisableKeepAlives = false
 
 			truth := true
 			staticCode := 204
@@ -38,6 +39,7 @@ var _ = Describe("Legacy Options", func() {
 						PassHostHeader:        &truth,
 						ProxyWebSockets:       &truth,
 						Timeout:               &timeout,
+						DisableKeepAlives:     legacyOpts.LegacyUpstreams.DisableKeepAlives,
 					},
 					{
 						ID:                    "/bar",
@@ -48,6 +50,7 @@ var _ = Describe("Legacy Options", func() {
 						PassHostHeader:        &truth,
 						ProxyWebSockets:       &truth,
 						Timeout:               &timeout,
+						DisableKeepAlives:     legacyOpts.LegacyUpstreams.DisableKeepAlives,
 					},
 					{
 						ID:                    "static://204",
@@ -60,6 +63,7 @@ var _ = Describe("Legacy Options", func() {
 						PassHostHeader:        nil,
 						ProxyWebSockets:       nil,
 						Timeout:               nil,
+						DisableKeepAlives:     legacyOpts.LegacyUpstreams.DisableKeepAlives,
 					},
 				},
 			}
@@ -145,6 +149,7 @@ var _ = Describe("Legacy Options", func() {
 		proxyWebSockets := true
 		flushInterval := Duration(5 * time.Second)
 		timeout := Duration(5 * time.Second)
+		disableKeepAlives := true
 
 		// Test cases and expected outcomes
 		validHTTP := "http://foo.bar/baz"
@@ -157,6 +162,7 @@ var _ = Describe("Legacy Options", func() {
 			ProxyWebSockets:       &proxyWebSockets,
 			FlushInterval:         &flushInterval,
 			Timeout:               &timeout,
+			DisableKeepAlives:     disableKeepAlives,
 		}
 
 		// Test cases and expected outcomes
@@ -170,6 +176,7 @@ var _ = Describe("Legacy Options", func() {
 			ProxyWebSockets:       &proxyWebSockets,
 			FlushInterval:         &flushInterval,
 			Timeout:               &timeout,
+			DisableKeepAlives:     disableKeepAlives,
 		}
 
 		validFileWithFragment := "file:///var/lib/website#/bar"
@@ -182,6 +189,7 @@ var _ = Describe("Legacy Options", func() {
 			ProxyWebSockets:       &proxyWebSockets,
 			FlushInterval:         &flushInterval,
 			Timeout:               &timeout,
+			DisableKeepAlives:     disableKeepAlives,
 		}
 
 		validStatic := "static://204"
@@ -197,6 +205,7 @@ var _ = Describe("Legacy Options", func() {
 			ProxyWebSockets:       nil,
 			FlushInterval:         nil,
 			Timeout:               nil,
+			DisableKeepAlives:     false,
 		}
 
 		invalidStatic := "static://abc"
@@ -212,6 +221,7 @@ var _ = Describe("Legacy Options", func() {
 			ProxyWebSockets:       nil,
 			FlushInterval:         nil,
 			Timeout:               nil,
+			DisableKeepAlives:     false,
 		}
 
 		invalidHTTP := ":foo"
@@ -226,6 +236,7 @@ var _ = Describe("Legacy Options", func() {
 					ProxyWebSockets:               proxyWebSockets,
 					FlushInterval:                 time.Duration(flushInterval),
 					Timeout:                       time.Duration(timeout),
+					DisableKeepAlives:             disableKeepAlives,
 				}
 
 				upstreams, err := legacyUpstreams.convert()
