@@ -1,9 +1,7 @@
 package cookies
 
 import (
-	"net/http"
 	"testing"
-	"time"
 
 	"github.com/oauth2-proxy/oauth2-proxy/v7/pkg/logger"
 	. "github.com/onsi/ginkgo/v2"
@@ -28,8 +26,3 @@ func TestProviderSuite(t *testing.T) {
 	RegisterFailHandler(Fail)
 	RunSpecs(t, "Cookies")
 }
-
-func testCookieExpires(exp time.Time) string {
-	var buf [len(http.TimeFormat)]byte
-	return string(exp.UTC().AppendFormat(buf[:0], http.TimeFormat))
-}
