@@ -6,11 +6,10 @@ import (
 	"fmt"
 	"net/http"
 	"net/url"
+	"slices"
 	"strings"
 	"time"
 
-	"golang.org/x/exp/slices"
-
 	"github.com/bitly/go-simplejson"
 
 	"github.com/oauth2-proxy/oauth2-proxy/v7/pkg/apis/options"
