@@ -4,6 +4,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"maps"
 	"net/http"
 	"net/url"
 	"path"
@@ -31,7 +32,8 @@ var _ Provider = (*GitHubProvider)(nil)
 
 const (
 	githubProviderName = "GitHub"
-	githubDefaultScope = "user:email"
+	githubDefaultScope = "user:email read:org"
+	orgTeamSeparator   = ":"
 )
 
 var (
@@ -114,9 +116,6 @@ func (p *GitHubProvider) makeGitHubAPIEndpoint(endpoint string, params *url.Valu
 func (p *GitHubProvider) setOrgTeam(org, team string) {
 	p.Org = org
 	p.Team = team
-	if org != "" || team != "" {
-		p.Scope += " read:org"
-	}
 }
 
 // setRepo configures the target repository and optional token to use
@@ -132,10 +131,19 @@ func (p *GitHubProvider) setUsers(users []string) {
 
 // EnrichSession updates the User & Email after the initial Redeem
 func (p *GitHubProvider) EnrichSession(ctx context.Context, s *sessions.SessionState) error {
-	err := p.getEmail(ctx, s)
-	if err != nil {
+	// Construct user info JSON from multiple GitHub API endpoints to have a more detailed session state
+	if err := p.getOrgAndTeam(ctx, s); err != nil {
 		return err
 	}
+
+	if err := p.checkRestrictions(ctx, s); err != nil {
+		return err
+	}
+
+	if err := p.getEmail(ctx, s); err != nil {
+		return err
+	}
+
 	return p.getUser(ctx, s)
 }
 
@@ -144,170 +152,107 @@ func (p *GitHubProvider) ValidateSession(ctx context.Context, s *sessions.Sessio
 	return validateToken(ctx, p, s.AccessToken, makeGitHubHeader(s.AccessToken))
 }
 
-func (p *GitHubProvider) hasOrg(ctx context.Context, accessToken string) (bool, error) {
+func (p *GitHubProvider) hasOrg(s *sessions.SessionState) error {
 	// https://developer.github.com/v3/orgs/#list-your-organizations
+	var orgs []string
 
-	var orgs []struct {
-		Login string `json:"login"`
-	}
-
-	type orgsPage []struct {
-		Login string `json:"login"`
-	}
-
-	pn := 1
-	for {
-		params := url.Values{
-			"per_page": {"100"},
-			"page":     {strconv.Itoa(pn)},
+	for _, group := range s.Groups {
+		if !strings.Contains(group, ":") {
+			orgs = append(orgs, group)
 		}
-
-		endpoint := p.makeGitHubAPIEndpoint("/user/orgs", &params)
-
-		var op orgsPage
-		err := requests.New(endpoint.String()).
-			WithContext(ctx).
-			WithHeaders(makeGitHubHeader(accessToken)).
-			Do().
-			UnmarshalInto(&op)
-		if err != nil {
-			return false, err
-		}
-
-		if len(op) == 0 {
-			break
-		}
-
-		orgs = append(orgs, op...)
-		pn++
 	}
 
 	presentOrgs := make([]string, 0, len(orgs))
 	for _, org := range orgs {
-		if p.Org == org.Login {
-			logger.Printf("Found Github Organization: %q", org.Login)
-			return true, nil
+		if p.Org == org {
+			logger.Printf("Found Github Organization:%q", org)
+			return nil
 		}
-		presentOrgs = append(presentOrgs, org.Login)
+		presentOrgs = append(presentOrgs, org)
 	}
 
 	logger.Printf("Missing Organization:%q in %v", p.Org, presentOrgs)
-	return false, nil
+	return errors.New("user is missing required organization")
 }
 
-func (p *GitHubProvider) hasOrgAndTeam(ctx context.Context, accessToken string) (bool, error) {
-	// https://developer.github.com/v3/orgs/teams/#list-user-teams
-
-	var teams []struct {
-		Name string `json:"name"`
-		Slug string `json:"slug"`
-		Org  struct {
-			Login string `json:"login"`
-		} `json:"organization"`
+func (p *GitHubProvider) hasOrgAndTeam(s *sessions.SessionState) error {
+	type orgTeam struct {
+		Org  string `json:"org"`
+		Team string `json:"team"`
 	}
 
-	type teamsPage []struct {
-		Name string `json:"name"`
-		Slug string `json:"slug"`
-		Org  struct {
-			Login string `json:"login"`
-		} `json:"organization"`
-	}
+	var presentOrgTeams []orgTeam
 
-	pn := 1
-	last := 0
-	for {
-		params := url.Values{
-			"per_page": {"100"},
-			"page":     {strconv.Itoa(pn)},
+	for _, group := range s.Groups {
+		if strings.Contains(group, orgTeamSeparator) {
+			ot := strings.Split(group, orgTeamSeparator)
+			presentOrgTeams = append(presentOrgTeams, orgTeam{ot[0], ot[1]})
 		}
-
-		endpoint := p.makeGitHubAPIEndpoint("/user/teams", &params)
-
-		// bodyclose cannot detect that the body is being closed later in requests.Into,
-		// so have to skip the linting for the next line.
-		// nolint:bodyclose
-		result := requests.New(endpoint.String()).
-			WithContext(ctx).
-			WithHeaders(makeGitHubHeader(accessToken)).
-			Do()
-		if result.Error() != nil {
-			return false, result.Error()
-		}
-
-		if last == 0 {
-			// link header may not be obtained
-			// When paging is not required and all data can be retrieved with a single call
-
-			// Conditions for obtaining the link header.
-			// 1. When paging is required (Example: When the data size is 100 and the page size is 99 or less)
-			// 2. When it exceeds the paging frame (Example: When there is only 10 records but the second page is called with a page size of 100)
-
-			// link header at not last page
-			// <https://api.github.com/user/teams?page=1&per_page=100>; rel="prev", <https://api.github.com/user/teams?page=1&per_page=100>; rel="last", <https://api.github.com/user/teams?page=1&per_page=100>; rel="first"
-			// link header at last page (doesn't exist last info)
-			// <https://api.github.com/user/teams?page=3&per_page=10>; rel="prev", <https://api.github.com/user/teams?page=1&per_page=10>; rel="first"
-
-			link := result.Headers().Get("Link")
-			rep1 := regexp.MustCompile(`(?s).*\<https://api.github.com/user/teams\?page=(.)&per_page=[0-9]+\>; rel="last".*`)
-			i, converr := strconv.Atoi(rep1.ReplaceAllString(link, "$1"))
-
-			// If the last page cannot be taken from the link in the http header, the last variable remains zero
-			if converr == nil {
-				last = i
-			}
-		}
-
-		var tp teamsPage
-		if err := result.UnmarshalInto(&tp); err != nil {
-			return false, err
-		}
-		if len(tp) == 0 {
-			break
-		}
-
-		teams = append(teams, tp...)
-
-		if pn == last {
-			break
-		}
-		if last == 0 {
-			break
-		}
-
-		pn++
 	}
 
 	var hasOrg bool
+
 	presentOrgs := make(map[string]bool)
 	var presentTeams []string
-	for _, team := range teams {
-		presentOrgs[team.Org.Login] = true
-		if p.Org == team.Org.Login {
+
+	for _, ot := range presentOrgTeams {
+		presentOrgs[ot.Org] = true
+
+		if strings.EqualFold(p.Org, ot.Org) {
 			hasOrg = true
-			ts := strings.Split(p.Team, ",")
-			for _, t := range ts {
-				if t == team.Slug {
-					logger.Printf("Found Github Organization:%q Team:%q (Name:%q)", team.Org.Login, team.Slug, team.Name)
-					return true, nil
+
+			teams := strings.Split(p.Team, ",")
+			for _, team := range teams {
+				if strings.EqualFold(strings.TrimSpace(team), ot.Team) {
+					logger.Printf("Found Github Organization/Team:%q/%q", ot.Org, ot.Team)
+					return nil
 				}
 			}
-			presentTeams = append(presentTeams, team.Slug)
+			presentTeams = append(presentTeams, ot.Team)
 		}
 	}
+
 	if hasOrg {
 		logger.Printf("Missing Team:%q from Org:%q in teams: %v", p.Team, p.Org, presentTeams)
-	} else {
-		var allOrgs []string
-		for org := range presentOrgs {
-			allOrgs = append(allOrgs, org)
-		}
-		logger.Printf("Missing Organization:%q in %#v", p.Org, allOrgs)
+		return errors.New("user is missing required team")
 	}
-	return false, nil
+
+	logger.Printf("Missing Organization:%q in %#v", p.Org, maps.Keys(presentOrgs))
+	return errors.New("user is missing required organization")
 }
 
-func (p *GitHubProvider) hasRepo(ctx context.Context, accessToken string) (bool, error) {
+func (p *GitHubProvider) hasTeam(s *sessions.SessionState) error {
+	var teams []string
+
+	for _, group := range s.Groups {
+		if strings.Contains(group, orgTeamSeparator) {
+			teams = append(teams, strings.TrimSpace(group))
+		}
+	}
+
+	var presentTeams = make([]string, 0, len(teams))
+
+	for _, ot := range teams {
+		allowedTeams := strings.Split(p.Team, ",")
+		for _, team := range allowedTeams {
+			if !strings.Contains(team, orgTeamSeparator) {
+				logger.Printf("Please use fully qualified team names (org:team-slug) if you omit the organisation. Current Team name: %s", team)
+				return errors.New("team name is invalid")
+			}
+
+			if strings.EqualFold(strings.TrimSpace(team), ot) {
+				logger.Printf("Found Github Organization/Team:%s", ot)
+				return nil
+			}
+		}
+		presentTeams = append(presentTeams, ot)
+	}
+
+	logger.Printf("Missing Team:%q in teams: %v", p.Team, presentTeams)
+	return errors.New("user is missing required team")
+}
+
+func (p *GitHubProvider) hasRepoAccess(ctx context.Context, accessToken string) error {
 	// https://developer.github.com/v3/repos/#get-a-repository
 
 	type permissions struct {
@@ -328,13 +273,18 @@ func (p *GitHubProvider) hasRepo(ctx context.Context, accessToken string) (bool,
 		WithHeaders(makeGitHubHeader(accessToken)).
 		Do().
 		UnmarshalInto(&repo)
+
 	if err != nil {
-		return false, err
+		return err
 	}
 
 	// Every user can implicitly pull from a public repo, so only grant access
 	// if they have push access or the repo is private and they can pull
-	return repo.Permissions.Push || (repo.Private && repo.Permissions.Pull), nil
+	if repo.Permissions.Push || (repo.Private && repo.Permissions.Pull) {
+		return nil
+	}
+
+	return errors.New("user doesn't have repository access")
 }
 
 func (p *GitHubProvider) hasUser(ctx context.Context, accessToken string) (bool, error) {
@@ -393,39 +343,8 @@ func (p *GitHubProvider) getEmail(ctx context.Context, s *sessions.SessionState)
 		Verified bool   `json:"verified"`
 	}
 
-	// If usernames are set, check that first
-	verifiedUser := false
-	if len(p.Users) > 0 {
-		var err error
-		verifiedUser, err = p.hasUser(ctx, s.AccessToken)
-		if err != nil {
-			return err
-		}
-		// org and repository options are not configured
-		if !verifiedUser && p.Org == "" && p.Repo == "" {
-			return errors.New("missing github user")
-		}
-	}
-	// If a user is verified by username options, skip the following restrictions
-	if !verifiedUser {
-		if p.Org != "" {
-			if p.Team != "" {
-				if ok, err := p.hasOrgAndTeam(ctx, s.AccessToken); err != nil || !ok {
-					return err
-				}
-			} else {
-				if ok, err := p.hasOrg(ctx, s.AccessToken); err != nil || !ok {
-					return err
-				}
-			}
-		} else if p.Repo != "" && p.Token == "" { // If we have a token we'll do the collaborator check in GetUserName
-			if ok, err := p.hasRepo(ctx, s.AccessToken); err != nil || !ok {
-				return err
-			}
-		}
-	}
-
 	endpoint := p.makeGitHubAPIEndpoint("/user/emails", nil)
+
 	err := requests.New(endpoint.String()).
 		WithContext(ctx).
 		WithHeaders(makeGitHubHeader(s.AccessToken)).
@@ -476,7 +395,6 @@ func (p *GitHubProvider) getUser(ctx context.Context, s *sessions.SessionState)
 	return nil
 }
 
-// isVerifiedUser
 func (p *GitHubProvider) isVerifiedUser(username string) bool {
 	for _, u := range p.Users {
 		if username == u {
@@ -485,3 +403,166 @@ func (p *GitHubProvider) isVerifiedUser(username string) bool {
 	}
 	return false
 }
+
+func (p *GitHubProvider) checkRestrictions(ctx context.Context, s *sessions.SessionState) error {
+	// If a user is verified by username options, skip the following restrictions
+	if ok, err := p.checkUserRestriction(ctx, s); err != nil || ok {
+		return err
+	}
+
+	var err error
+	switch {
+	case p.Org != "" && p.Team != "":
+		err = p.hasOrgAndTeam(s)
+	case p.Org != "":
+		err = p.hasOrg(s)
+	case p.Team != "":
+		err = p.hasTeam(s)
+	}
+
+	if err != nil {
+		return err
+	}
+
+	if p.Org == "" && p.Repo != "" && p.Token == "" {
+		// If we have a token we'll do the collaborator check
+		return p.hasRepoAccess(ctx, s.AccessToken)
+	}
+
+	return nil
+}
+
+func (p *GitHubProvider) checkUserRestriction(ctx context.Context, s *sessions.SessionState) (bool, error) {
+	if len(p.Users) == 0 {
+		return false, nil
+	}
+
+	verifiedUser, err := p.hasUser(ctx, s.AccessToken)
+	if err != nil {
+		return verifiedUser, err
+	}
+
+	// org and repository options are not configured
+	if !verifiedUser && p.Org == "" && p.Repo == "" {
+		return false, errors.New("missing github user")
+	}
+
+	return verifiedUser, nil
+}
+
+func (p *GitHubProvider) getOrgAndTeam(ctx context.Context, s *sessions.SessionState) error {
+	err := p.getOrgs(ctx, s)
+	if err != nil {
+		return err
+	}
+
+	return p.getTeams(ctx, s)
+}
+
+func (p *GitHubProvider) getOrgs(ctx context.Context, s *sessions.SessionState) error {
+
+	type Organization struct {
+		// Support for Github organizations
+		// https://docs.github.com/en/rest/orgs/orgs#list-organizations-for-the-authenticated-user
+		Login string `json:"login,omitempty"`
+		// Support for Gitea organizations
+		// https://docs.gitea.com/api/1.24/#tag/organization/operation/orgGetAll
+		Name string `json:"name,omitempty"`
+	}
+
+	pn := 1
+	for {
+		params := url.Values{
+			"per_page": {"100"},
+			"page":     {strconv.Itoa(pn)},
+		}
+
+		endpoint := p.makeGitHubAPIEndpoint("/user/orgs", &params)
+
+		var orgs []Organization
+		err := requests.New(endpoint.String()).
+			WithContext(ctx).
+			WithHeaders(makeGitHubHeader(s.AccessToken)).
+			Do().
+			UnmarshalInto(&orgs)
+		if err != nil {
+			return err
+		}
+
+		if len(orgs) == 0 {
+			break
+		}
+
+		for _, org := range orgs {
+			var orgName string
+			if len(org.Login) > 0 {
+				orgName = org.Login
+				logger.Printf("Member of Github Organization: %q", orgName)
+			} else {
+				orgName = org.Name
+				logger.Printf("Member of Gitea Organization: %q", orgName)
+			}
+
+			s.Groups = append(s.Groups, orgName)
+		}
+		pn++
+	}
+
+	return nil
+}
+
+func (p *GitHubProvider) getTeams(ctx context.Context, s *sessions.SessionState) error {
+	// https://docs.github.com/en/rest/teams/teams?#list-user-teams
+	type Team struct {
+		Name string `json:"name"`
+		Slug string `json:"slug"`
+		Org  struct {
+			Login string `json:"login"`
+			Name  string `json:"name"`
+		} `json:"organization"`
+	}
+
+	pn := 1
+	for {
+		params := url.Values{
+			"per_page": {"100"},
+			"page":     {strconv.Itoa(pn)},
+		}
+
+		endpoint := p.makeGitHubAPIEndpoint("/user/teams", &params)
+
+		var teams []Team
+		err := requests.New(endpoint.String()).
+			WithContext(ctx).
+			WithHeaders(makeGitHubHeader(s.AccessToken)).
+			Do().
+			UnmarshalInto(&teams)
+		if err != nil {
+			return err
+		}
+
+		if len(teams) == 0 {
+			break
+		}
+
+		for _, team := range teams {
+			var orgName, teamName string
+
+			if len(team.Org.Login) > 0 {
+				orgName = team.Org.Login
+				teamName = team.Slug
+				logger.Printf("Member of Github Organization/Team: %q/%q", orgName, teamName)
+			} else {
+				orgName = team.Org.Name
+				teamName = team.Name
+				logger.Printf("Member of Gitea Organization/Team: %q/%q", orgName, teamName)
+			}
+
+			s.Groups = append(s.Groups, fmt.Sprintf("%s%s%s", orgName, orgTeamSeparator, teamName))
+		}
+
+		pn++
+	}
+
+	return nil
+}
