@@ -31,7 +31,7 @@ type GitLabProvider struct {
 var _ Provider = (*GitLabProvider)(nil)
 
 // NewGitLabProvider initiates a new GitLabProvider
-func NewGitLabProvider(p *ProviderData, opts options.GitLabOptions) (*GitLabProvider, error) {
+func NewGitLabProvider(p *ProviderData, opts options.Provider) (*GitLabProvider, error) {
 	p.setProviderDefaults(providerDefaults{
 		name: gitlabProviderName,
 	})
@@ -40,15 +40,15 @@ func NewGitLabProvider(p *ProviderData, opts options.GitLabOptions) (*GitLabProv
 		p.Scope = gitlabDefaultScope
 	}
 
-	oidcProvider := NewOIDCProvider(p, options.OIDCOptions{InsecureSkipNonce: false})
+	oidcProvider := NewOIDCProvider(p, opts.OIDCConfig)
 
 	provider := &GitLabProvider{
 		OIDCProvider:    oidcProvider,
 		oidcRefreshFunc: oidcProvider.RefreshSession,
 	}
-	provider.setAllowedGroups(opts.Group)
+	provider.setAllowedGroups(opts.GitLabConfig.Group)
 
-	if err := provider.setAllowedProjects(opts.Projects); err != nil {
+	if err := provider.setAllowedProjects(opts.GitLabConfig.Projects); err != nil {
 		return nil, fmt.Errorf("could not configure allowed projects: %v", err)
 	}
 
@@ -167,7 +167,7 @@ func (p *GitLabProvider) getUserinfo(ctx context.Context, s *sessions.SessionSta
 	var userinfo gitlabUserinfo
 	err := requests.New(userinfoURL.String()).
 		WithContext(ctx).
-		SetHeader("Authorization", "Bearer "+s.AccessToken).
+		SetHeader("Authorization", tokenTypeBearer+" "+s.AccessToken).
 		Do().
 		UnmarshalInto(&userinfo)
 	if err != nil {
@@ -246,7 +246,7 @@ func (p *GitLabProvider) getProjectInfo(ctx context.Context, s *sessions.Session
 
 	err := requests.New(fmt.Sprintf("%s%s", endpointURL.String(), url.QueryEscape(project))).
 		WithContext(ctx).
-		SetHeader("Authorization", "Bearer "+s.AccessToken).
+		SetHeader("Authorization", tokenTypeBearer+" "+s.AccessToken).
 		Do().
 		UnmarshalInto(&projectInfo)
 	if err != nil {
